
# q-expansion of newform 1617.4.a.p, downloaded from the LMFDB on 17 June 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField, ZZ
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K([c/ZZ(den) for c in num]) for num, den in basis_data]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 1617
weight = 4
poly_data = [-64, 108, -11, -28, 0, 1]

# The entries in the following list give a basis for the
# coefficient ring in terms of a root of the defining polynomial above.
# Each line consists of the coefficients of the numerator, and a denominator.
basis_data  = [[[1, 0, 0, 0, 0], 1], [[0, 1, 0, 0, 0], 1], [[-11, -1, 1, 0, 0], 1], [[64, -17, -26, 0, 1], 2], [[74, -57, -28, 2, 1], 2]]

hecke_ring_character_values = None
aps_data = [[1, -1, 0, 0, 0], [-3, 0, 0, 0, 0], [-1, 1, 0, 0, -1], [0, 0, 0, 0, 0], [11, 0, 0, 0, 0], [-22, -1, 1, -1, -1], [-29, -14, 5, 7, 2], [-21, 5, 0, -4, -3], [-4, -6, 0, 8, -4], [10, 35, 13, 7, -1], [59, 4, 1, -17, -8], [-6, 17, -13, -19, 1], [-77, -34, 7, -7, -24], [80, -2, 16, -16, 4], [-15, -55, 22, 10, -11], [308, 16, 12, 8, -26], [-160, -91, -51, 11, 47], [-132, -40, -22, -46, 12], [-126, 139, 21, 3, 13], [-142, 4, -14, 14, -20], [-189, -23, 24, 36, 23], [-28, -88, 0, -16, -52], [-205, 64, 69, -13, 16], [222, -164, 16, -20, -8], [178, 188, -34, 14, -2], [54, -90, -20, 32, -16], [773, -190, -77, -11, 28], [120, -339, 29, -21, -33], [-110, 58, 60, 40, 72], [672, -120, -14, -114, -88], [-828, 304, -14, -90, -54], [-1151, -88, -55, 55, 22], [716, -186, -190, -94, -28], [-177, 164, 55, 49, -86], [976, -425, 47, 97, 103], [856, -344, -70, -82, -58], [905, -174, -11, -157, -16], [222, -37, -207, -33, 37], [-142, 70, -166, -82, -44], [-1160, 530, 136, -104, -14], [1464, 76, 40, 160, 108], [-869, -120, -129, 109, 68], [696, -440, 26, -10, 110], [1286, 212, -102, -38, -14], [-476, 76, -98, 86, 100], [1185, 238, 17, 55, 18], [744, 458, 286, -22, -234], [1383, 546, 17, 71, -20], [-1999, 538, -121, -151, -56], [-517, 944, -17, -3, -92], [1754, -428, 4, 128, -196], [-1510, -609, -419, -377, 1], [-621, -389, -38, -114, 77], [1784, 387, -493, -79, 113], [300, 1125, -143, 19, -39], [598, 873, -325, 81, -129], [-1703, 644, 31, 77, 134], [620, 109, 513, 159, -221], [58, 330, -370, -294, 6], [640, 683, 515, 457, 47], [-411, 277, 314, 150, 105], [76, -486, -72, -64, -298], [1687, 652, 363, 121, -234], [2529, -1684, -135, 455, -22], [-1320, -846, 206, 286, 200], [-1052, 548, -188, -528, 22], [-2364, 1440, 220, -68, -172], [436, 1040, 68, -192, -334], [-386, 492, 422, -194, -312], [-3212, 1291, 111, 261, 175], [2020, 391, -21, -147, -217], [1288, -176, 564, 524, -212], [5203, -430, -19, -45, -252], [1732, 1754, -378, -10, 380], [-2278, 509, -297, -251, 251], [6641, 1316, -143, -325, 74], [-174, 530, 282, -754, 306], [3133, 958, -789, -343, 294], [4104, 608, -168, 252, -166], [543, -1592, -251, 367, 206], [1442, -233, 517, -257, -491], [-676, 1323, -197, -55, 207], [-3878, -399, -793, -199, -201], [-2706, 1610, 234, -458, 630], [4588, 1765, -55, -157, -285], [2244, -636, -408, -184, 40], [220, 1206, 148, -180, 170], [3894, -2432, 58, -86, 410], [-5434, -1658, 94, 578, 470], [-10062, 867, -375, -533, -563], [26, 381, 715, 697, -217], [-1100, 460, 1218, -154, -70], [-6250, -320, -762, -626, -252], [-762, 2229, -277, 65, 267], [4146, 1421, 591, -563, -301], [4650, -3150, -1162, -38, 304], [-3407, -2524, 111, -743, 334], [4948, 377, -47, -317, 217], [6869, 2479, -1144, -24, -5], [3964, -262, 508, 788, 390], [-10514, -2676, 398, 658, -232], [-942, 411, 145, -1, 3], [-1439, 2966, -619, -517, 410], [5410, -728, 0, 144, 104], [-3490, 4472, -506, -22, -616], [5596, -1274, 146, -174, 984], [-2808, 3999, 1079, -119, -467], [7563, 1066, 279, 801, -824], [-8370, -688, -750, -218, -80], [-7039, 2889, -614, -754, -273], [18722, 1553, -1153, -599, 295], [-1176, -2150, -470, -318, -524], [10330, -832, -1604, -356, 516], [2512, -1096, -342, -466, -942], [-3162, 80, -230, 726, -240], [-4430, 3294, 868, 1232, 300], [3538, -126, 354, 342, 1448], [319, -5361, -610, -122, 767], [1490, -2850, 422, -1014, 126], [7608, 1417, 17, -717, 707], [1607, 1036, -413, -191, 932], [3812, -1400, -364, -128, 834], [-11696, -412, -618, -566, -1244], [-3652, 372, 2286, 434, -6], [4798, 1550, 1076, 332, -438], [-8414, -960, 564, -480, 544], [7598, -2081, -47, 43, -129], [1187, 3495, -1090, -1094, 499], [12619, 2590, 491, 69, 918], [5212, 4552, -766, -786, -300], [7774, -4822, -762, 498, -720], [13970, 2065, -1465, -1667, -89], [-1796, -1113, -1125, -711, 393], [6394, -783, 1299, 193, 241], [1881, -2946, -263, -513, 136], [-11559, -4437, -800, -836, 889], [4993, 609, 430, 1406, 711], [3215, -3525, -886, -2022, -513], [17, -3325, 964, -40, 857], [12698, -89, -559, -2137, -753], [7397, 2879, -848, -616, 2339], [-5288, -1639, 1497, 987, -1803], [4208, 1207, -1129, -175, -95], [7782, 2393, 507, -15, -309], [-6085, -4418, 1911, 145, -204], [-2981, -2903, 1936, -212, -415], [-19976, -3776, 930, 730, -288], [15101, -3300, -1041, 537, 1342], [7622, -5472, 498, 2182, -340], [9308, -3322, 108, 1620, 1896], [14154, 1482, 502, -102, -522], [-24662, 93, -1673, 49, 13], [-4094, 5513, 2055, 1565, 459], [16712, -1092, 2150, 858, 362], [-14292, 1864, 3128, 160, 48], [22280, 4414, -578, -502, -118], [12976, 26, 1256, -808, 1440], [-8108, 2971, 907, 1833, 663], [-11431, -4404, -957, -1483, 126], [3256, 2998, 172, 108, 334], [16908, -2490, 2302, 258, 1394], [-8858, 11, 101, 711, -945], [-15360, 1894, 494, 2330, 1562], [-3042, 4255, 2117, 63, -1903], [-6210, -1052, 50, 1066, 2322], [6647, 5984, -425, 753, 446], [10610, 2831, 113, 211, -1067], [-1906, -644, -72, -684, 1452], [13990, -7540, -984, -288, -556], [-10105, -1415, 1200, -328, -393], [12820, -5127, 545, 1263, -993], [-6531, 4640, 2315, 205, -2506], [-12768, -1008, 920, -1928, -144], [9148, 6354, -674, 550, -1800], [17789, 7082, 1231, 1177, -1392], [6580, 3132, 1986, 490, -8], [-1850, -2922, -1342, -1898, -1356], [-36146, -2452, 104, 608, 760], [1688, -7069, 135, 3121, -231], [2972, -563, -2487, -1229, 765], [23386, -121, 2537, 2715, -907], [-30339, -4418, 261, 2307, 174], [-724, 4960, 730, -406, -128], [-10943, -4102, -1239, 1303, -1248], [7746, 1140, -2106, -2190, -448], [-9369, 7657, 1784, -1388, 911], [7872, 5814, 2024, -2640, -1754], [37016, -2028, 858, -2426, -2510], [11788, -5406, 1902, 1174, -680], [15546, 7132, -1612, -204, -854], [-24280, 3924, -1762, -1222, -912], [-6224, -9125, -1621, 1613, 1465], [21930, -2813, -219, 2719, 517], [31957, -561, 1040, 1400, 1241], [-3890, 3545, 203, -2855, 555], [26464, 2639, -1921, -1979, 1615], [-5804, -6562, -282, 1878, 2512], [-24228, 13264, 2218, 746, 1000], [4227, 1776, 799, 357, 2296], [14502, 1518, 582, -238, 3268], [-5686, -5212, -472, 3012, -556], [-6860, -4649, -1005, 981, -3735], [18205, 12380, 2449, 1147, -2456], [159, -4463, 1664, 2796, 251], [-11352, -2536, -2716, -1808, -3360], [1981, -4132, 1851, 3113, 2374], [25221, -5528, 1869, -53, 1658], [-834, -921, 4237, -729, 601], [15452, -6732, -3942, -710, 1756], [-1776, 3026, -3100, -2564, -328], [19892, 4548, -242, -3806, 72], [41089, 1707, 2048, 404, 773], [12834, -11311, -1825, -7, -1857], [-17230, -7368, 2770, 2750, -2640], [-4570, 12626, 168, 2112, 3970], [-36004, 5084, -1922, -2582, -3340], [-4248, 7836, 2308, 2492, 3636], [-7223, 9150, -2205, -3703, -2266], [-874, -1996, 3056, 4440, -342], [-13870, 4682, 956, -3192, 2392], [7024, -5782, 1086, 254, -392], [-4620, -1969, 1347, 2897, 2593], [13012, 7748, 4968, 3988, 1982], [20454, -2875, -3045, 97, 4695], [40385, -2408, 701, -1813, -2478], [12374, -6920, 3016, 4744, 320], [1418, 7435, 1745, -2941, -389], [43022, -1214, 248, 32, 2658], [-9732, 2167, 4571, -699, 1393], [-9408, -2429, 1911, 5169, 3293], [-13524, 3264, -1586, 3910, 676], [-1261, -10673, -1774, -1502, 3459], [21972, -12746, -762, -5206, -2810], [14366, 12767, 37, -2693, -1529], [20108, 11578, -3108, -6212, -1708], [-9402, 6154, 840, 80, 1322], [-13082, -10994, 2076, -24, -3604], [-47696, 1583, -3205, -2003, -3653], [-4264, -624, 4418, 1446, 1546], [-1055, 3289, -4758, 1210, 3081], [9248, 10994, -2232, -1984, 932], [5825, -15444, -35, 851, -262], [8681, -18444, -1913, 4369, 1268], [1402, -11072, -2048, 156, 5836], [937, -6834, 4001, -1833, -504], [15102, -12604, -1050, 3554, 4380], [36873, 988, 3293, -781, -2890], [-43600, -7451, -827, -353, 3535], [-13366, 1136, -3398, 2430, -1960], [9238, -12467, -2621, -2539, -497], [-6860, -10132, -3966, -2410, 424], [-28597, 6728, 3255, 2717, 856], [4390, -358, 2996, 1852, 6], [29622, 19656, -3156, -6140, -1308], [25364, -3664, 750, 1866, -1632], [22282, -2720, -5098, -4198, 1028], [-40622, 1473, -5745, 489, -2563], [19205, 9327, 1768, 520, -2181], [-23157, -14133, -5214, -3078, 1313], [-47457, -8795, -4868, -5328, -905], [-31344, -4150, 3554, -3858, 1854], [80200, 584, 1324, -4100, -3064], [21024, 2972, -2686, -642, -3032], [4224, -2751, 3109, 867, -1347], [5778, 3571, 1629, -2189, 3279], [30402, 16836, -1000, 3224, 642], [-47708, 5312, -2022, -1994, -1840], [-11626, -16619, 1291, 2345, -315], [20322, 11392, 4042, 6318, 3616], [-23512, -4532, -3338, 4682, 2328], [-60115, -5450, -2175, -2649, 3040], [-30266, -1069, 8625, 1147, -2291], [20812, 5816, -7416, 132, 1002], [-28084, 564, 1182, 162, -3018], [-11193, -7887, 4280, 4264, -853], [6887, -6500, -4255, 1263, 340], [65600, 3528, -7644, -3476, 1024], [20656, 9400, 70, -6326, -1800], [1966, 10666, 2112, -2516, 1308], [18479, 11054, 583, -8679, 290], [18240, -5994, 8988, -2252, -412], [872, -12752, 2144, 2320, -976], [-2800, 3472, 3290, -1154, -338], [19036, 17273, 5213, -1393, -4415], [27471, -14987, -4032, -4956, -5023], [-7572, 5798, -1472, 1640, 6388], [-50074, -297, -891, -725, -6153], [-45986, 3304, -3684, 336, -2732], [-16307, 18836, -1145, -7215, 1648], [-28083, -8982, -9117, -3215, 2786], [-6588, 8922, 5202, 3682, 152], [-47936, -8708, -3096, 476, 1964], [52904, -3861, -2393, 3349, 35], [-31121, -6358, -4673, 337, -2974], [-24766, -14858, -4598, 4022, 2698], [-28262, -2472, 9994, 5314, 342], [8225, -2522, 5331, -2387, -2008], [8753, -4100, -11105, -4203, -10], [-16835, 18510, -1459, -3229, -5250], [-17499, -2159, 4970, 1478, 2977], [13161, -21377, -1032, 5800, -4039], [15520, -16887, -1867, 5591, -1671], [-12688, -10514, -2612, 4284, 6788], [-27076, -15994, 1704, 4448, 248], [-17326, 12511, -1791, -4193, -3783], [-13526, 10963, 2809, 5215, -2721], [52208, 2670, 272, 3520, 4532], [-68761, -5560, -5361, -8139, 4016], [68622, 3322, -3938, -6398, 484], [2706, 1132, 6506, 3178, -6814], [-59780, -6086, -4994, 6690, 1926], [36086, -16538, 726, -6254, -3246], [21136, 14363, 6935, -1751, -10717], [-78424, 3544, -532, -5852, -584], [34968, 7536, 1208, 288, -616], [-46007, 16472, 3167, -5059, 3954], [-2690, 2236, 3210, -5458, 436], [-23852, 8336, -1548, -3924, -1032], [7852, -6512, 1700, 440, 470], [-68962, -8772, -52, -3928, 248], [-90365, 5429, -776, 4968, 2829], [-38904, -7673, -77, -6199, -2153], [21304, -20704, 3858, -5682, -3368], [14114, 9796, 2610, -2378, 5668], [-37842, 14873, -3449, 1141, 2261], [-64256, 14574, 2896, -6792, 2390], [-115340, -5000, -2160, -1584, 600], [-30816, 18018, -2644, 316, -4112], [-30518, -10049, -4455, -9405, 2789], [-32464, 3519, 1671, 2361, -4225], [38312, 16186, -304, -312, -346], [-68662, 1122, -3002, 3538, 1206], [35604, 20469, -5995, -2225, 375], [22929, -3456, 2607, -431, 2080], [27982, -25386, 122, 1934, 6276], [-16198, -13903, 2467, 9513, -4489], [-14690, -21569, -6011, -4349, -3293], [63753, 1916, 10805, 4575, 1828], [-40670, -7178, -5064, -3604, 3036], [-31384, -192, -384, -916, 1464], [-57698, -10696, -5604, -6312, -3116], [40290, 6526, -956, -2048, 3748], [-64592, 6286, 3138, 1386, 3640], [19364, 22405, 4713, -1721, -7781], [-21378, -25874, 4228, -3856, -2220], [-23286, -1419, -3777, 1493, 10123], [89051, 24500, -7347, -2533, 1894], [-52716, 3384, 3396, -172, 1308], [29248, 808, -5508, 6708, -2092], [-23849, 8864, 10817, -725, -1514], [5991, -10880, 11731, -5659, 2958], [-3622, -26175, 267, -2663, -5995], [-59096, 30174, 6376, -2464, -3922], [23070, 2173, -889, -8555, -2717], [-26994, 27050, -2584, 5004, 4744], [-1991, -8518, -4021, 2329, 6454], [71813, -4831, 1366, -4494, 4161], [-41531, 1640, -1635, -7053, 6454], [-3270, 5479, 5233, 2555, -5785], [19702, -26378, 2110, 7138, -2308], [-84326, 18798, 6934, -350, 2798], [-40939, -3482, -13271, -3169, -2724], [33030, -40420, 1174, 6866, 6396], [-94366, -4651, 7447, 3785, -3549], [-38968, 2984, 10764, 4092, 48], [-10656, -22177, -1989, -8091, -9433], [-81432, -3398, 11950, 10438, 1436], [13940, -18294, 3262, -4902, -154], [19058, 34008, 160, -5024, -4104], [-50432, -31548, 4214, 1010, 5062], [36352, 15132, -1198, 3278, 474], [-7798, 6515, -327, 7235, 2869], [-33914, 31949, -2813, -9531, -2395], [-7188, -10424, 252, 4968, -240], [66544, -18657, -14033, -1887, 4415], [140498, 7487, 1969, 1487, 105], [36394, 95, -1603, -1885, 11617], [58441, 11509, 5726, -3794, 1067], [-88657, 18116, -1143, -1185, -4062], [43557, -5500, -1953, -6199, 3606], [57981, 14179, 9608, 6128, -3783], [20570, 5664, -8830, -9518, -3106], [73580, 14530, 2502, -5774, -9162], [-83798, 18171, -3999, 1287, -841], [151551, -1956, 5339, 8309, -2354], [-17365, -11439, -7440, 1252, 785], [-48176, -17637, -5177, -3631, 4537], [121699, -1076, 1933, -1301, -946], [-53092, -832, -4958, 1910, -1882], [-37111, 8083, 6968, 1012, -2997], [43208, -15255, 1073, 8211, 5553], [72471, 17268, -11175, -6629, 7950], [110456, 18134, -2456, 3424, -2124], [27234, 30994, 6898, -1506, -7496], [-37254, -4658, -5264, -1664, 5110], [-56588, 14774, -5706, -178, 8344], [25110, -48147, -4289, 2893, 4931], [-26358, 29343, -2187, -7425, -5497], [120972, -968, 2148, 5708, 5316], [-1628, 1883, 7115, -6991, 3079], [-6930, -26261, 4089, -3829, -5917], [4298, -2498, -6812, -11192, 1044], [-83660, 22636, 2102, 11418, 7846], [-67460, 24748, -6942, 7530, 4112], [-43664, -104, -2288, 3460, -7606], [-56248, 19119, 17023, 1585, -7311], [-25896, -7493, 4423, 9493, -945], [-42768, -27216, 262, -34, -8092], [126456, 22786, -762, -2302, 1270], [-42644, 27968, -2468, 4212, -6292], [-44975, 7568, -11885, -7247, -2274], [-5784, -1944, -15594, -16138, 424], [18576, -11308, -7268, -572, 6200], [32446, 36170, 1014, 4266, -2752], [62280, 6112, 5002, 7662, 1840], [-118519, -10751, -5426, -106, 4679], [-93450, -15351, 1463, -4847, 5867], [-30321, -2344, -1091, 11675, 11232], [-23404, 31644, 14398, 8166, -1772], [-26920, -32960, 3810, 3326, 6366], [-35930, -27530, 8464, 8452, -2144], [59550, 39715, 533, 2031, -4737], [-40650, 21952, -3952, -5888, 6130], [-112213, 744, -14055, -7129, -5876], [23140, -16508, -4094, -5218, -12120], [-51263, -33547, 9002, 9478, -3149], [-49296, 6596, -1902, 15062, 7338], [30324, -23017, 855, 9133, 727], [37536, 13294, 7142, 738, -15466], [103440, 3824, 16880, 8952, 2936]]
